#include <oxstd.h>
#include <oxprob.h>
#include "C:\\myfiles\\ivqr\\ox\\rq.ox"

//**********************************************************
// [bhat,g,A,ghat]=iqr(y,D,X,Z,tau,A,GLOB)
// Note, all inputs are required.  If there are no covariates or instruments, pass <>.
//  y - dependent variable
//  D - endogenous RHS variable (Current program requires D be an n X 1 vector)
//  X - covariates
//  Z - excluded instruments
//  tau - quantile index (tau in (0,1))
//  A - grid search region.  Note that A must be a column vector.
//  GLOB - 1 for preprocessing. 0 for no pre-processing.  Must be specified.
// Output.
//  bhat - coefficient estimates and standard errors.
//  g - norm(gamma(alpha))
//  A - grid search region
//  ghat - gamma(alpha)
//      NOTE: After estimation, plot(alpha,g) graphs objective function.
//*****************************************************************************
iqr(y,D,X,Z,tau,A,GLOB){
	
	decl n, i, bhat, g, I, est, ghat, W;

	n=rows(y);

	est=zeros(columns(Z~X)+1,rows(A));
	g=zeros(rows(A),1);
	bhat=zeros(columns(X)+2,1);
	ghat=zeros(columns(Z),1);
	
	if(columns(Z)==columns(D)){					  // Just-id case
		for (i=0; i<rows(A); ++i)				  // Grid search
		{
			println(i);
			est[][i]=rq(Z~X,y-(A[i][0])*D,tau,GLOB);
			g[i][0]=sqrt(sumsqrc(est[0][i]));
		}
		I=mincindex(g);							  // Find index of minimum of norm(gammahat)
		bhat[0][0]=A[I][0];						  // Coef of endogenous regressor
		bhat[1:][0]=est[1:][I];					  // Coefs of exogenous regressors
		ghat=est[0][I];							  // Estimated gamma
	}
	else{										  // Over-id case
		W = Z'*Z-(Z'*(X~ones(n,1)))*invert((X~ones(n,1))'*(X~ones(n,1)))*((X~ones(n,1))'*Z);
						// "Reasonable" weighting matrix
	    for (i=0; i<rows(A); ++i)				  // Grid search
		{
			println(i);
			est[][i]=rq(Z~X,y-(A[i][0])*D,tau,GLOB);
			g[i][0]=sqrt((est[0:columns(Z)-1][i])'*W*(est[0:columns(Z)-1][i]));
		}
		I=mincindex(g);									 // Index of min(norm(gammahat))
		bhat[0][0]=A[I][0];								 // Coef of endogenous regressor
		bhat[1:][0]=rq(X,y-(bhat[0][0])*D,tau,GLOB);	 // Coefs of exogenous regressors
		ghat=est[0:columns(Z)-1][I];					 // Estimated gammas
	}

	return{bhat,g,A,ghat};
}
