#include <oxstd.h>
#include <oxprob.h>
#include "C:\\myfiles\\ivqr\\ox\\rq.ox"
#include "C:\\myfiles\\ivqr\\ox\\vciqr.ox"
#include "C:\\myfiles\\ivqr\\ox\\vcqr.ox"
#include "C:\\myfiles\\ivqr\\ox\\iqrproc.ox"

//**********************************************************
// iqr(y,D,X,Z,A,GLOB)
// Estimates empirical quantile regression process for data.  Note that this algorithm
// 		collapses Z to a single instrument, the linear projection of D on {X,Z}.
//		The algorithm is currently implemented for a single RHS endogenous variable only.
// Note, all inputs are required.  If there are no covariates or instruments, pass [].
//  y - dependent variable
//  D - endogenous RHS variable (Current program requires D be an n X 1 vector)
//  X - covariates
//  Z - excluded instruments
//  A - grid search region.  Note that A must be a column vector.
//  GLOB - 1 for preprocessing. 0 for no pre-processing.  Must be specified.
// Output.
//  bhat - coefficient estimates and standard errors for IQR estimates for tau in [.1,.9]
//		spaced at .01 intervals.  (2 columns per tau (coef, std. error))
//  ghat - gamma(alpha).  As above, 1 column per tau.
//  bhat_qr - coefficient estimates and standard errors for QR estimates for tau in [.1,.9]
//		spaced at .01 intervals.  (2 columns per tau (coef, std. error))
//  stats - vector containing test statistics from process tests (See iqrproc.ox for more detail.)
//  crvals - matrix containing bootstrap test statistics, used to construct
//      critical values for the process tests.  (See iqrproc.ox for more detail.)
//*****************************************************************************
iqr(y,D,X,Z,A,GLOB){
	
	decl n, i, bhat, g, I, est, ghat, W, ZEST, tol, check, out, J, temp, bhat_qr, H, j,
		C1, C2, C3, grid, tau, result, PROCTEST, crvals, stats, small;

	n=rows(y);
	tol = 10^-3;
	
	tau = .01*<10:90>;
	J = zeros(81*(columns(X)+2),columns(X)+2);
	H = zeros(81*(columns(X)+2),columns(X)+2);

	bhat = zeros(columns(X)+2,162);
	bhat_qr = zeros(columns(X)+2,162);
	ghat = zeros(columns(Z),81);

	W = Z~X~ones(n,1);
	ZEST = W*invert(W'*W)*(W'*D);

	W = ZEST~X;

	est = zeros(columns(Z~X)+1,rows(A));
	g=10*ones(rows(A),1);
	check = 1;
	i = 0;	

	while(i<rows(A) && check>tol)
	{
//		println(i);
		est[][i] = rq(W,y-A[i][0]*D,.5,GLOB);
		g[i][0] = sqrt(sumsqrc(est[0][i]));
		check = g[i][0];
		++i;
	}
			
	I=mincindex(g);
	ghat[0][40]=est[0][I];
	out = kcov_iqr(A[I][0] | est[1:][I],.5,y,D,X,ZEST);
	bhat[][80:81] = out[0];
	J[40*columns(J):41*columns(J)-1][] = out[2];

	temp = rq(D~X,y,.5,1);
	out = kcov_qr(temp,.5,y,D~X);
	bhat_qr[][80:81] = out[0];
	H[40*columns(J):41*columns(J)-1][] = out[2];

	for (i = 1; i < 41; ++i)
		{
		small = (1/10)*bhat[0][2*(41-i)+1]*rann(1,1);
		println(50-i);
		C1 = bhat[0][2*(41-i)] + small - (3/2)*bhat[0][2*(41-i)+1];
		C2 = (3/8)*bhat[0][2*(41-i)+1];
		C3 = bhat[0][2*(41-i)] + small + (3/2)*bhat[0][2*(41-i)+1];
		grid = range(C1,C3,C2);
		grid = grid';

		est = zeros(columns(Z~X)+1,rows(grid));
		g=10*ones(rows(grid),1);
		check = 1;
		j = 0;	

		while(j<rows(grid) && check>tol)
		{
//			println(j);
			est[][j] = rq(W,y-grid[j][0]*D,.5-.01*i,GLOB);
			g[j][0] = sqrt(sumsqrc(est[0][j]));
			check = g[j][0];
			++j;
		}
			
		I=mincindex(g);
		ghat[0][40-i]=est[0][I];
		out = kcov_iqr(grid[I][0] | est[1:][I],.5-.01*i,y,D,X,ZEST);
		bhat[][2*(40-i):2*(40-i)+1] = out[0];
		J[(40-i)*columns(J):(41-i)*columns(J)-1][] = out[2];

		temp = rq(D~X,y,.5-.01*i,1);
		out = kcov_qr(temp,.5-.01*i,y,D~X);
		bhat_qr[][2*(40-i):2*(40-i)+1] = out[0];
		H[(40-i)*columns(J):(41-i)*columns(J)-1][] = out[2];

		small = (1/10)*bhat[0][2*(39+i)+1]*rann(1,1);
		println(50+i);
		C1 = bhat[0][2*(39+i)] + small - (3/2)*bhat[0][2*(39+i)+1];
		C2 = (3/8)*bhat[0][2*(39+i)+1];
		C3 = bhat[0][2*(39+i)] + small + (3/2)*bhat[0][2*(39+i)+1];
		grid = range(C1,C3,C2);
		grid = grid';

		est = zeros(columns(Z~X)+1,rows(grid));
		g=10*ones(rows(grid),1);
		check = 1;
		j = 0;	

		while(j<rows(grid) && check>tol)
		{
//			println(j);
			est[][j] = rq(W,y-grid[j][0]*D,.5+.01*i,GLOB);
			g[j][0] = sqrt(sumsqrc(est[0][j]));
			check = g[j][0];
			++j;
		}
			
		I=mincindex(g);
		ghat[0][40+i]=est[0][I];
		out = kcov_iqr(grid[I][0] | est[1:][I],.5+.01*i,y,D,X,ZEST);
		bhat[][2*(40+i):2*(40+i)+1] = out[0];
		J[(40+i)*columns(J):(41+i)*columns(J)-1][] = out[2];

		temp = rq(D~X,y,.5+.01*i,1);
		out = kcov_qr(temp,.5+.01*i,y,D~X);
		bhat_qr[][2*(40+i):2*(40+i)+1] = out[0];
		H[(40+i)*columns(J):(41+i)*columns(J)-1][] = out[2];
	}
		

	PROCTEST = iqrproc(5*(rows(y)^(2/5)),250,tau,J,H,bhat,bhat_qr,y,D,X,ZEST);

  	stats = PROCTEST[0]~PROCTEST[1]~PROCTEST[4]~PROCTEST[5]~PROCTEST[8]~PROCTEST[9]~PROCTEST[12]~PROCTEST[13];
	crvals = PROCTEST[2]'~PROCTEST[3]'~PROCTEST[6]'~PROCTEST[7]'~PROCTEST[10]'~PROCTEST[11]'~PROCTEST[14]'~PROCTEST[15]';

	result = new array[5];

	result[0] = bhat;
	result[1] = ghat;
	result[2] = bhat_qr;
	result[3] = stats;
	result[4] = crvals;
	
	return(result);
}
